# Filename:     tb_moore_example.do
# Filetype:     Modelsim Script File
# Date:         21 apri; 2016
# Update:       
# Description:  Script File For Automatic Simulation
# Author:       J. op den Brouw
# State:        Demo
# Error:        -
# Version:      1.0
# Copyright:    (c)2016, De Haagse Hogeschool
#

# Transcript window
transcript on

# Recreate the RTL work directory
if {[file exists rtl_work]} {
	vdel -lib rtl_work -all
}
vlib rtl_work
vmap work rtl_work

# Find out if we're started through Quartus or by hand
# (or by using an exec in the Tcl window in Quartus).
# Quartus has the annoying property that it will start
# Modelsim from a directory called "simulation/modelsim".
# The design and the testbench are located in the project
# root, so we've to compensate for that.
if [ string match "*simulation/modelsim" [pwd] ] { 
	set prefix "../../"
	puts "Running Modelsim from Quartus..."
} else {
	set prefix ""
	puts "Running Modelsim..."
}

# Compile the VHDL description and testbench,
# please note that the design and its testbench is located
# in the project root, but the simulator start in directory
# <project_root>/simulation/modelsim, so we have to compensate
# for that.
vcom -2008 -work work ${prefix}moore_example.vhd
vcom -2008 -work work ${prefix}tb_moore_example.vhd

# Start simulation with 1 ns time step
vsim -t 1ns -L rtl_work -L work -voptargs="+acc" tb_moore_example

# Log all signals in the design, good if the number
# of signals is small.
add log -r *

# Add all toplevel signals
# Add a number of signals of the simulated design
add list *
if {[find signals dut/current_state] != ""} {
	add list dut/current_state
}
if {[find signals dut/next_state] != ""} {
	add list dut/next_state
}
if {[find signals dut/state] != ""} {
	add list dut/state
}

# Add all toplevel signals
# Add a number of signals of the simulated design
add wave -divider "SYSTEM"
add wave areset
add wave clk
add wave -divider "Inputs"
add wave x
add wave y
add wave -divider "Internals"
if {[find signals dut/current_state] != ""} {
	add wave dut/current_state
}
if {[find signals dut/next_state] != ""} {
	add wave dut/next_state
}
if {[find signals dut/state] != ""} {
	add wave dut/state
}
add wave -divider "Outputs"
add wave z

# Open Structure, Signals (waveform) and List window
view structure
view list
view wave

# Run simulation for 160 ns
run 160 ns

# Fill up the waveform in the window
wave zoom full