-- Filename:     tb_moore_example.vhd
-- Filetype:     VHDL testbench code
-- Date:         21 april 2016
-- Update:       -
-- Description:  An example of a testbench
-- Author:       J. op den Brouw
-- State:        demo
-- Error:        -
-- Version:      1.0
-- Copyright:    (c)2016, De Haagse Hogeschool

-- This VHDL code describes an example of a testbench
-- for the VHDL description of the Moore machine in
-- this example project.

library ieee;                                               
use ieee.std_logic_1164.all;                                

-- Testbench has empty entity
entity tb_moore_example is
end entity tb_moore_example;

-- The architecture
architecture testbench of tb_moore_example is
-- Some timing parameters
constant Tperiod : time := 10 ns;
constant small_delay : time := 1 ns;

-- Top level signals to be monitored in the simulator
signal clk : std_logic;
signal areset : std_logic;
signal x : std_logic;
signal y : std_logic;
signal z : std_logic;

-- Component description of the Moore machine
component moore_example
    port ( clk :    in std_logic;
           areset : in std_logic;
           x :      in std_logic;
           y :      in std_logic;
           z :      out std_logic
    );
end component;

-- Choose one of the architectures...
--for dut : moore_example use entity work.moore_example(fsm_2proc);
--for dut : moore_example use entity work.moore_example(fsm_3proc);
for dut : moore_example use entity work.moore_example(fsm_2proc_alt);

begin
	
    -- Instantiation of the Mealy machine
    dut : moore_example
    port map (clk => clk, areset => areset, x => x, y => y, z => z);

    -- The clock, creates a never ending clock
    clockgen: process is
    begin
        clk <= '0';
        wait for Tperiod/2;
		  clk <= '1';
        wait for Tperiod/2;
    end process clockgen;

    -- Data generation, including asynchronous reset  
    datagen: process is
    begin
        -- Set known value on the input 
        x <= '0';
        y <= '0';

        -- Activate reset
        areset <= '1';
        wait for 2*Tperiod;

        -- Deactivate reset
        areset <= '0';
        wait for 2*Tperiod;
		  
		  -- Wait on posedge clock
        wait until clk = '1';
        wait for small_delay;

        -- Supply the machine with data
        x <= '1';
        y <= '0';
        wait until clk = '1';
        wait for small_delay;

        x <= '1';
        y <= '1';
        wait until clk = '1';
        wait for small_delay;

        x <= '1';
        y <= '1';
        wait until clk = '1';
        wait for small_delay;

        x <= '1';
        y <= '0';
        wait until clk = '1';
        wait for small_delay;

        x <= '1';
        y <= '1';
        wait until clk = '1';
        wait for small_delay;

        x <= '0';
        y <= '1';
        wait until clk = '1';
        wait for small_delay;

        x <= '1';
        y <= '1';
        wait until clk = '1';
        wait for small_delay;

        x <= '1';
        y <= '1';
        wait until clk = '1';
        wait for small_delay;

        wait;
    end process datagen;

end testbench;
